@@ -47,6 +47,8 @@ from ..requirements import (
     NodeModuleRequirement,
     NodePackageRequirement,
     LibraryRequirement,
+    BoostComponentRequirement,
+    StaticLibraryRequirement,
     RubyFileRequirement,
     XmlEntityRequirement,
     SprocketsFileRequirement,
@@ -68,6 +70,8 @@ from ..requirements import (
     CertificateAuthorityRequirement,
     LibtoolRequirement,
     VagueDependencyRequirement,
+    PerlPreDeclaredRequirement,
+    IntrospectionTypelibRequirement,
 )
 
 
@@ -115,6 +119,36 @@ class AptRequirement(Requirement):
                 return True
         return False
 
+    def satisfied_by(self, binaries, version):
+        def binary_pkg_matches(entry, binary):
+            # TODO(jelmer): check versions
+            if entry['name'] == binary['Package']:
+                return True
+            for provides_top in PkgRelation.parse_relations(
+                    binary.get('Provides', '')):
+                for provides in provides_top:
+                    if entry['name'] == provides['name']:
+                        return True
+            return False
+
+        for rel in self.relations:
+            for entry in rel:
+                if any(binary_pkg_matches(entry, binary) for binary in binaries):
+                    break
+            else:
+                return False
+        return True
+
+
+def resolve_perl_predeclared_req(apt_mgr, req):
+    try:
+        req = req.lookup_module()
+    except KeyError:
+        logging.warning(
+            'Unable to map predeclared function %s to a perl module', req.name)
+        return None
+    return resolve_perl_module_req(apt_mgr, req)
+
 
 def find_package_names(
     apt_mgr: AptManager, paths: List[str], regex: bool = False, case_insensitive=False
@@ -146,7 +180,6 @@ def python_spec_to_apt_rels(pkg_name, specs):
     else:
         rels = []
         for spec in specs:
-            deb_version = Version(spec[1])
             if spec[0] == "~=":
                 # PEP 440: For a given release identifier V.N , the compatible
                 # release clause is approximately equivalent to the pair of
@@ -155,32 +188,26 @@ def python_spec_to_apt_rels(pkg_name, specs):
                 parts.pop(-1)
                 parts[-1] = str(int(parts[-1]) + 1)
                 next_maj_deb_version = Version(".".join(parts))
+                deb_version = Version(spec[1])
                 rels.extend(
-                    [
-                        {"name": pkg_name, "version": (">=", deb_version)},
-                        {"name": pkg_name, "version": ("<<", next_maj_deb_version)},
-                    ]
-                )
+                    [[{"name": pkg_name, "version": (">=", deb_version)}],
+                     [{"name": pkg_name, "version": ("<<", next_maj_deb_version)}]])
             elif spec[0] == "!=":
-                rels.extend(
-                    [
-                        {"name": pkg_name, "version": (">>", deb_version)},
-                        {"name": pkg_name, "version": ("<<", deb_version)},
-                    ]
-                )
+                deb_version = Version(spec[1])
+                rels.extend([
+                    [{"name": pkg_name, "version": (">>", deb_version)}],
+                    [{"name": pkg_name, "version": ("<<", deb_version)}]])
             elif spec[1].endswith(".*") and spec[0] == "==":
                 s = spec[1].split(".")
                 s.pop(-1)
                 n = list(s)
                 n[-1] = str(int(n[-1]) + 1)
                 rels.extend(
-                    [
-                        {"name": pkg_name, "version": (">=", Version(".".join(s)))},
-                        {"name": pkg_name, "version": ("<<", Version(".".join(n)))},
-                    ]
-                )
+                    [[{"name": pkg_name, "version": (">=", Version(".".join(s)))}],
+                     [{"name": pkg_name, "version": ("<<", Version(".".join(n)))}]])
             else:
                 c = {">=": ">=", "<=": "<=", "<": "<<", ">": ">>", "==": "="}[spec[0]]
+                deb_version = Version(spec[1])
                 rels.append([{"name": pkg_name, "version": (c, deb_version)}])
         return rels
 
@@ -282,6 +309,17 @@ def get_package_for_python_module(apt_mgr, module, python_version, specs):
 vague_map = {
     "the Gnu Scientific Library": "libgsl-dev",
     "the required FreeType library": "libfreetype-dev",
+    "the Boost C++ libraries": "libboost-dev",
+
+    # TODO(jelmer): Support resolving virtual packages
+    "PythonLibs": "libpython3-dev",
+    "ZLIB": "libz3-dev",
+    "Osmium": "libosmium2-dev",
+    "glib": "libglib2.0-dev",
+
+    # TODO(jelmer): For Python, check minimum_version and map to python 2 or python 3
+    "Python": "libpython3-dev",
+    "Lua": "liblua5.4-dev",
 }
 
 
@@ -289,9 +327,22 @@ def resolve_vague_dep_req(apt_mgr, req):
     name = req.name
     options = []
     if name in vague_map:
-        options.append(AptRequirement.simple(vague_map[name]))
+        options.append(AptRequirement.simple(vague_map[name], minimum_version=req.minimum_version))
     for x in req.expand():
         options.extend(resolve_requirement_apt(apt_mgr, x))
+    # Try even harder
+    if not options:
+        options.extend(find_reqs_simple(
+            apt_mgr,
+            [
+                posixpath.join("/usr/lib", ".*", "pkgconfig", re.escape(req.name) + "-.*\\.pc"),
+                posixpath.join("/usr/lib/pkgconfig", re.escape(req.name) + "-.*\\.pc")
+            ],
+            regex=True,
+            case_insensitive=True,
+            minimum_version=req.minimum_version
+        ))
+
     return options
 
 
@@ -393,9 +444,9 @@ def resolve_php_package_req(apt_mgr, req):
 
 def resolve_r_package_req(apt_mgr, req):
     paths = [
-        posixpath.join("/usr/lib/R/site-library/.*/R/%s$" % re.escape(req.package))
+        posixpath.join("/usr/lib/R/site-library", req.package, "DESCRIPTION")
     ]
-    return find_reqs_simple(apt_mgr, paths, regex=True)
+    return find_reqs_simple(apt_mgr, paths, minimum_version=req.minimum_version)
 
 
 def resolve_node_module_req(apt_mgr, req):
@@ -426,6 +477,14 @@ def resolve_library_req(apt_mgr, req):
     return find_reqs_simple(apt_mgr, paths, regex=True)
 
 
+def resolve_static_library_req(apt_mgr, req):
+    paths = [
+        posixpath.join("/usr/lib/%s$" % re.escape(req.filename)),
+        posixpath.join("/usr/lib/.*/%s$" % re.escape(req.filename)),
+    ]
+    return find_reqs_simple(apt_mgr, paths, regex=True)
+
+
 def resolve_ruby_file_req(apt_mgr, req):
     paths = [posixpath.join("/usr/lib/ruby/vendor_ruby/%s.rb" % req.filename)]
     reqs = find_reqs_simple(apt_mgr, paths, regex=False)
@@ -485,7 +544,7 @@ def resolve_cmake_file_req(apt_mgr, req):
 
 
 def resolve_haskell_package_req(apt_mgr, req):
-    path = "/var/lib/ghc/package\\.conf\\.d/%s-.*\\.conf" % re.escape(req.deps[0][0])
+    path = "/var/lib/ghc/package\\.conf\\.d/%s-.*\\.conf" % re.escape(req.package)
     return find_reqs_simple(apt_mgr, [path], regex=True)
 
 
@@ -543,18 +602,22 @@ def resolve_libtool_req(apt_mgr, req):
 
 
 def resolve_perl_module_req(apt_mgr, req):
-    DEFAULT_PERL_PATHS = ["/usr/share/perl5"]
+    DEFAULT_PERL_PATHS = ["/usr/share/perl5", "/usr/lib/.*/perl5/.*", "/usr/lib/.*/perl-base"]
 
     if req.inc is None:
         if req.filename is None:
-            paths = [posixpath.join(inc, req.relfilename) for inc in DEFAULT_PERL_PATHS]
+            paths = [posixpath.join(inc, re.escape(req.module.replace('::', '/') + '.pm')) for inc in DEFAULT_PERL_PATHS]
+            regex = True
         elif not posixpath.isabs(req.filename):
-            return False
+            paths = [posixpath.join(inc, re.escape(req.filename)) for inc in DEFAULT_PERL_PATHS]
+            regex = True
         else:
             paths = [req.filename]
+            regex = False
     else:
+        regex = False
         paths = [posixpath.join(inc, req.filename) for inc in req.inc]
-    return find_reqs_simple(apt_mgr, paths, regex=False)
+    return find_reqs_simple(apt_mgr, paths, regex=regex)
 
 
 def resolve_perl_file_req(apt_mgr, req):
@@ -620,6 +683,12 @@ def resolve_ca_req(apt_mgr, req):
     return [AptRequirement.simple("ca-certificates")]
 
 
+def resolve_introspection_typelib_req(apt_mgr, req):
+    return find_reqs_simple(
+        apt_mgr, [r'/usr/lib/.*/girepository-.*/%s-.*\.typelib' % re.escape(req.library)],
+        regex=True)
+
+
 def resolve_apt_req(apt_mgr, req):
     # TODO(jelmer): This should be checking whether versions match as well.
     for package_name in req.package_names():
@@ -628,10 +697,17 @@ def resolve_apt_req(apt_mgr, req):
     return [req]
 
 
+def resolve_boost_component_req(apt_mgr, req):
+    return find_reqs_simple(
+        apt_mgr, ["/usr/lib/.*/libboost_%s" % re.escape(req.name)],
+        regex=True)
+
+
 APT_REQUIREMENT_RESOLVERS = [
     (AptRequirement, resolve_apt_req),
     (BinaryRequirement, resolve_binary_req),
     (VagueDependencyRequirement, resolve_vague_dep_req),
+    (PerlPreDeclaredRequirement, resolve_perl_predeclared_req),
     (PkgConfigRequirement, resolve_pkg_config_req),
     (PathRequirement, resolve_path_req),
     (CHeaderRequirement, resolve_c_header_req),
@@ -647,6 +723,7 @@ APT_REQUIREMENT_RESOLVERS = [
     (NodeModuleRequirement, resolve_node_module_req),
     (NodePackageRequirement, resolve_node_package_req),
     (LibraryRequirement, resolve_library_req),
+    (StaticLibraryRequirement, resolve_static_library_req),
     (RubyFileRequirement, resolve_ruby_file_req),
     (XmlEntityRequirement, resolve_xml_entity_req),
     (SprocketsFileRequirement, resolve_sprockets_file_req),
@@ -668,6 +745,8 @@ APT_REQUIREMENT_RESOLVERS = [
     (PythonPackageRequirement, resolve_python_package_req),
     (CertificateAuthorityRequirement, resolve_ca_req),
     (CargoCrateRequirement, resolve_cargo_crate_req),
+    (IntrospectionTypelibRequirement, resolve_introspection_typelib_req),
+    (BoostComponentRequirement, resolve_boost_component_req),
 ]
 
 
@@ -683,11 +762,20 @@ def resolve_requirement_apt(apt_mgr, req: Requirement) -> List[AptRequirement]:
     raise NotImplementedError(type(req))
 
 
+def default_tie_breakers(session):
+    from ..debian.udd import popcon_tie_breaker
+    from ..debian.build_deps import BuildDependencyTieBreaker
+    return [
+        BuildDependencyTieBreaker.from_session(session),
+        popcon_tie_breaker,
+        ]
+
+
 class AptResolver(Resolver):
     def __init__(self, apt, tie_breakers=None):
         self.apt = apt
         if tie_breakers is None:
-            tie_breakers = []
+            tie_breakers = default_tie_breakers(apt.session)
         self.tie_breakers = tie_breakers
 
     def __str__(self):
